<!doctype html>

<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{% block title %} __temp_theme_dir__ {% endblock %}</title>
    <link rel="stylesheet" href="{{ url_for('__temp_theme_dir__.static', filename='css/example__css.css') }}">
    <script src="{{ url_for('__temp_theme_dir__.static', filename='js/example__js.js') }}"></script>
</head>

<body>
    <img src="{{ url_for('__temp_theme_dir__.static', filename='img/Flask-BigApp-Logo.png') }}" alt="logo">
    <p>__temp_theme_dir__ theme</p>
    {% include "__temp_theme_dir__/includes/menu.html" %}
    {% block global %}
    
    {% endblock %}
    {% include "__temp_theme_dir__/includes/footer.html" %}
</body>

</html>
